@@ -5,10 +5,14 @@
  *  Find the start and end macro markers.  In btween we must find the
  *  "autogen" and "template" keywords, followed by any suffix specs.
  *
- *  This module processes the "pseudo" macro
+ *  This module processes the "pseudo" macro.
  *
+ * @addtogroup autogen
+ * @{
+ */
+/*
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (C) 1992-2013 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -384,9 +388,13 @@ copy_mark(char const * text, char * marker, size_t * ret_ct)
 /**
  *  Using a finite state machine, scan over the tokens that make up the
  *  "pseudo macro" at the start of every template.
+ *
+ *  @param[in,out]  text    text of template
+ *  @param[in]      fname   name of template file
+ *  @returns the address of the byte following the pseudo macro
  */
 LOCAL char const *
-loadPseudoMacro(char const * text, char const * fname)
+load_pseudo_mac(char const * text, char const * fname)
 {
     char const * pzBadness;
 #   define BAD_MARKER(t) { pzBadness = t; goto abort_load; }
@@ -486,7 +494,9 @@ loadPseudoMacro(char const * text, char const * fname)
 #   undef BAD_MARKER
     return NULL;
 }
-/*
+/**
+ * @}
+ *
  * Local Variables:
  * mode: C
  * c-file-style: "stroustrup"
