@@ -30,8 +30,8 @@
 
 if test -z "$mainpid"
 then
-    . ${top_srcdir}/config/bootstrap.shlib
-    . ${top_builddir}/config/shdefs
+    . ${top_srcdir:-..}/config/bootstrap.shlib
+    . ${top_builddir:-..}/config/shdefs
 fi
 
 set_defaults()
@@ -131,12 +131,13 @@ make_exprini()
     exec 3> expr.cfg
     cat >&3 <<- _EOConfig_
 	defs-to-get gfunc
-	template    snarf
+	template    snarf.tpl
 	srcfile
-	assign      two-phase = yes
-	assign      group     = ag
+	assign      two-phase   = yes
+	assign      group       = ag
+	assign      addtogroup  = "autogen"
 	output      expr.def
-	subblock    exparg    = arg_name,arg_desc,arg_optional,arg_list
+	subblock    exparg      = arg_name,arg_desc,arg_optional,arg_list
 	_EOConfig_
 
     test ${#DEBUG_ENABLED} -gt 0 && \
@@ -161,15 +162,39 @@ make_exprini()
 #
 make_directive()
 {
-    cat > directive.cfg <<- _EOConfig_
-	defs-to-get    directive
-	template       directive
-	output         directive.def
-	linenum
-	_EOConfig_
+    : ${AG_TIMEOUT=10}
+    to=--timeout=`expr $AG_TIMEOUT '*' 3`
+    dlist=$(
+        sed -n '/^doDir_invalid/d
+		/^doDir_[a-z]*(/{;s@(.*@@;s@^doDir_@@;p;}' defDirect.c | \
+            sort)
+    test -f directive.def && rm -f directive.def
+    {
+        cat <<- _EOF_
+		AutoGen Definitions str2enum;
+		prefix = dir;
+		type   = '';
+		length = '';
+		addtogroup = 'autogen';
+		add-on-text = { ao-file = enum-header;
+		   ao-text  = "/* #ident, #let and #pragma are ignored */\n"
+		              "#define doDir_let        ignore_directive\n"
+		              "#define doDir_ident      ignore_directive\n"
+		              "#define doDir_pragma     ignore_directive\n\n"
+		              "#define doDir_endshell   bad_dirv_ctx\n"
+		              "#define doDir_elif       bad_dirv_ctx\n"
+		              "#define doDir_endmac     bad_dirv_ctx\n"
+		              "#define doDir_ifndef     doDir_ifdef"; };
+		dispatch = {
+		   d-nam = 'doDir_%s';
+		   d-ret = 'char *';
+		   d-arg = 'char * scan_next';
+		};
+		_EOF_
+        echo cmd = $(echo $dlist | tr ' ' ',') \;
+    } > directive.def
 
-    ${GDexe} load=directive.cfg ${srcdir}/defDirect.c
-    run_ag directive -t30 directive.def
+    run_ag dirtv directive.def
     rmlist=${rmlist}\ directive.cfg
 }
 
@@ -179,7 +204,9 @@ make_directive()
 #
 make_texi()
 {
+    : ${AG_TIMEOUT=10}
     eopt="-Tagtexi-cmd.tpl -DLEVEL=chapter -binvoke-autogen"
+    eopt=${eopt}\ --timeout=`expr $AG_TIMEOUT '*' 3`
     run_ag texi ${eopt} ${srcdir}/opts.def
 }
 
@@ -264,7 +291,7 @@ do
     expr.h | expr.def | expr.ini )
         dispatch exprini ;;
 
-    directive.def | directive.h )
+    directive.h )
         dispatch directive ;;
 
     autogen.texi | autogen.menu )
