@@ -1,7 +1,7 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for agen5
- * Generated Fri Jan  4 21:29:43 PST 2013
+ * Generated Sat Feb 23 07:49:03 PST 2013
  */
 #ifndef AGEN5_PROTO_H_GUARD
 #define AGEN5_PROTO_H_GUARD 1
@@ -37,7 +37,7 @@ wrap_up_depends(void);
  *  Extracted from agInit.c
  */
 static void
-initialize(int arg_ct, char** arg_vec);
+initialize(int arg_ct, char ** arg_vec);
 
 static void
 config_dep(tOptions * opts, tOptDesc * od);
@@ -106,8 +106,8 @@ ao_strdup (char const * str);
 /*
  *  Extracted from defDirect.c
  */
-static char*
-processDirective(char* pzScan);
+static char *
+processDirective(char * scan);
 
 /*
  *  Extracted from defFind.c
@@ -142,6 +142,9 @@ new_def_ent(void);
 static void
 print_ent(def_ent_t * de);
 
+static void
+delete_ent(def_ent_t * de);
+
 static def_ent_t *
 number_and_insert_ent(char * name, char const * idx_str);
 
@@ -151,6 +154,12 @@ read_defs(void);
 static void
 unload_defs(void);
 
+/*
+ *  Extracted from expExtract.c
+ */
+static char *
+load_file(char const * fname);
+
 /*
  *  Extracted from expGuile.c
  */
@@ -225,7 +234,7 @@ static char const *
 do_suffix(char const * const text, char const * fname, int lineNo);
 
 static char const *
-loadPseudoMacro(char const * text, char const * fname);
+load_pseudo_mac(char const * text, char const * fname);
 
 /*
  *  Extracted from scmStrings.c
