@@ -2,11 +2,9 @@
 
 ## mdoc2man.sh -- script to convert mdoc-isms to man-isms
 ##
-## Time-stamp:      "2011-09-25 10:31:27 bkorb"
-##
 ##  This file is part of AutoOpts, a companion to AutoGen.
 ##  AutoOpts is free software.
-##  AutoOpts is Copyright (c) 1992-2012 Bruce Korb - all rights reserved
+##  AutoOpts is Copyright (C) 1992-2012 Bruce Korb - all rights reserved
 ##
 ##  AutoOpts is available under any one of two licenses.  The license
 ##  in use must be one of these two and the choice is under the control
@@ -18,11 +16,11 @@
 ##   The Modified Berkeley Software Distribution License
 ##      See the file "COPYING.mbsd"
 ##
-##  These files have the following md5sums:
+##  These files have the following sha256 sums:
 ##
-##  43b91e8ca915626ed3818ffb1b71248b COPYING.gplv3
-##  06a1a2e4760c90ea5e1dad8dfaac4d39 COPYING.lgplv3
-##  66a5cedaf62c4b2637025f049f9b826f COPYING.mbsd
+##  8584710e9b04216a394078dc156b781d0b47e1729104d666658aecef8ee32e95  COPYING.gplv3
+##  4379e7444a0e2ce2b12dd6f5a52a27a4d02d39d247901d3285c88cf0d37f477b  COPYING.lgplv3
+##  13aa749a5b0a454917a944ed8fffc530b784f5ead522b1aacaf4ec8aa55a6239  COPYING.mbsd
 
 ## This "library" converts mdoc-isms into man-isms.  It gets included
 ## by the man page template at the point where mdoc-isms might start appearing
@@ -199,25 +197,38 @@ do_optional() {
     local text='['
     while test $# -gt 0
     do
-	m1="$1"
+        # Ns may be fun...
+        m1="$1"
         case "X$1" in
-	'X...' | 'X\*' )
-            text=${text}' "\fI'${1}'\fR"'
-            shift
+        X\\\& )
+            # The general escape sequence is: \&
+            text="${text} ${1}"
             ;;
+
+        X[!0-9a-zA-Z]* )
+            # This one should be a generic punctuation check
+            # Punctuation is any of the 10 chars: .,:;()[]?!  (but
+            # more are actually used and accepted) and is output in
+            # the default font.
+            #
+            text="${text} ${1}"
+            ;;
+
         XAr | XCm )
             text=${text}' "\fI'${2}'\fR"'
-            shift 2 || had_no_arg "$m1"
+            shift
             ;;
+
         XFl )
             text=${text}' \fB-'${2}'\fR'
-            shift 2 || had_no_arg "$m1"
+            shift
             ;;
-        * ) text="${text} \"$2\""
-	    m1="$1"
-            shift 2 || had_no_arg "$m1"
+
+        * ) # Grab subsequent non-keywords and punctuation, too
+            text="${text} $1"
             ;;
         esac
+        shift || die "shift failed after $m1"
     done
     echo "${text} ]"
 }
@@ -248,11 +259,11 @@ do_NmName() {
 
     if test $# -gt 0
     then case "$1" in
-	[A-Za-z]* )
-	    NmName=$1
-	    shift
-	    ;;
-	esac
+        [A-Za-z]* )
+            NmName=$1
+            shift
+            ;;
+        esac
 
         test $# -gt 0 && NmNameSfx=" $*"
     fi
@@ -268,8 +279,8 @@ do_line() {
     .Op' '* )        do_optional  ;;
     .Fl' '* )        do_flag      ;;
     .Ar' '* )        do_arg       ;;
-    .Nm' '* )	     do_NmName	  ;;
-    .Nm     )	     do_NmName	  ;;
+    .Nm' '* )        do_NmName    ;;
+    .Nm     )        do_NmName    ;;
     * )              echo "$line" ;;
     esac
     return 0
@@ -279,6 +290,7 @@ do_line() {
 
 easy_fixes='
 s/^\.Sh/.SH/
+s/^\.Ss/.SS/
 s/^\.Em/.I/
 s/^\.Pp/.PP/
 s/^.in *\\-/.in -/
